from .track import track_page_view


def page_view_tracking_middleware(get_response):
    def middleware(request):
        response = get_response(request)

        track_page_view(request)

        return response

    return middleware



import logging
import requests
import uuid

from django.conf import settings


logger = logging.getLogger(__name__)

# For an overview of all of the available parameters available to be sent to GA:
# https://developers.google.com/analytics/devguides/collection/protocol/v1/parameters

API_VERSION = "1"

GOOGLE_ANALYTICS_ENDPOINT = "https://www.google-analytics.com/collect"
DEBUG_GOOGLE_ANALYTICS_ENDPOINT = "https://www.google-analytics.com/debug/collect"


def build_tracking_data(request, additional_data):
    data = {
        "v": API_VERSION,  # API Version.
        "tid": settings.GA_MEASUREMENT_PROTOCOL_UA,  # Tracking aID / Property ID.
        "cid": str(
            uuid.uuid4()
        ),  # This needs to be cid not uid or your events won't register in the behaviour section
        "uip": request.META.get("REMOTE_ADDR"),  # User ip override
        "aip": "1",  # Anonymise user ip
        "ua": request.META.get("HTTP_USER_AGENT"),  # User agent override
        "dr": request.META.get("HTTP_REFERER"),  # Document referrer
        "dl": request.build_absolute_uri(),  # Document location URL
        **additional_data,
    }

    return data


def send_tracking_data(tracking_data):
    if not getattr(settings, "GA_MEASUREMENT_PROTOCOL_TRACK_EVENTS", False):
        return

    if getattr(settings, "GA_MEASUREMENT_PROTOCOL_DEBUG", False):
        response = requests.post(
            DEBUG_GOOGLE_ANALYTICS_ENDPOINT,
            params=tracking_data,
        )
        logger.debug("Tracking response: %s", response.json())
    else:
        response = requests.post(
            GOOGLE_ANALYTICS_ENDPOINT,
            params=tracking_data,
        )

    return response


def track_event(request, category, action, label=None, value=None):
    event_data = {
        "t": "event",
        "ec": category,
        "ea": action,
    }
    if label:
        event_data["el"] = label

    if value:
        event_data["ev"] = value

    data = build_tracking_data(request, event_data)

    return send_tracking_data(data)


def track_page_view(request):
    page_view_data = {
        "t": "pageview",
    }

    data = build_tracking_data(request, page_view_data)

    return send_tracking_data(data)



DATABASES = {
    "default": {"ENGINE": "django.db.backends.sqlite3", "NAME": ":memory:"},
}

INSTALLED_APPS = [
    "django_ga_measurement_protocol.tests.testapp",
]

SECRET_KEY = "abcdefghijklmnopqrstuvwxyz"



from unittest import mock

from django.test import override_settings, TestCase
from django.urls import reverse


@override_settings(
    MIDDLEWARE=[
        "django_ga_measurement_protocol.middleware.page_view_tracking_middleware"
    ],
    ROOT_URLCONF="django_ga_measurement_protocol.tests.urls",
)
class PageViewTrackingMiddlewareTestCase(TestCase):
    @mock.patch("django_ga_measurement_protocol.middleware.track_page_view")
    def test_page_view_called_on_view(self, mock_track_page_view):
        response = self.client.get(reverse("test-middleware"))
        mock_track_page_view.assert_called_once_with(response.wsgi_request)



import requests_mock
import uuid

from unittest import mock

from django.test import override_settings, RequestFactory, TestCase

from ..track import (
    API_VERSION,
    build_tracking_data,
    DEBUG_GOOGLE_ANALYTICS_ENDPOINT,
    GOOGLE_ANALYTICS_ENDPOINT,
    send_tracking_data,
    track_event,
    track_page_view,
)


FAKE_GA_ID = "ua-12345-1"


def get_request_data(request):
    return request.qs


@override_settings(
    GA_MEASUREMENT_PROTOCOL_TRACK_EVENTS=True,
    GA_MEASUREMENT_PROTOCOL_UA=FAKE_GA_ID,
)
class TrackingTestCase(TestCase):
    def setUp(self):
        self.factory = RequestFactory()

    @mock.patch("django_ga_measurement_protocol.track.uuid")
    def test_build_tracking_data(self, mock_uuid):
        mock_uuid_value = uuid.uuid4()
        mock_uuid.uuid4.return_value = mock_uuid_value

        url = "/test-url/"
        remote_addr = "127.0.0.1"
        http_user_agent = "user-agent-string"
        http_referer = "http://example.com"

        request = self.factory.get(
            url,
            REMOTE_ADDR=remote_addr,
            HTTP_USER_AGENT=http_user_agent,
            HTTP_REFERER=http_referer,
        )

        tracking_data = build_tracking_data(
            request,
            {
                "test-key": "test-value",
            },
        )

        self.assertEqual(
            tracking_data,
            {
                "aip": "1",
                "cid": str(mock_uuid_value),
                "dl": f"http://testserver{url}",
                "dr": http_referer,
                "tid": FAKE_GA_ID,
                "ua": http_user_agent,
                "uip": remote_addr,
                "v": API_VERSION,
                "test-key": "test-value",
            },
        )

    @requests_mock.Mocker()
    def test_send_tracking_data(self, mock_requests):
        m = mock_requests.post(
            GOOGLE_ANALYTICS_ENDPOINT,
        )

        send_tracking_data({"test-key": "test-value"})
        request = m.request_history[0]
        data = get_request_data(request)

        self.assertEqual(
            data,
            {"test-key": ["test-value"]},
        )

    @override_settings(
        GA_MEASUREMENT_PROTOCOL_TRACK_EVENTS=False,
    )
    @requests_mock.Mocker()
    def test_send_tracking_data_settings_disabled(self, mock_requests):
        m = mock_requests.post(
            GOOGLE_ANALYTICS_ENDPOINT,
        )

        send_tracking_data({"test-key": "test-value"})
        self.assertEqual(len(m.request_history), 0)

    @override_settings(
        GA_MEASUREMENT_PROTOCOL_DEBUG=True,
    )
    @requests_mock.Mocker()
    @mock.patch("django_ga_measurement_protocol.track.logger")
    def test_send_tracking_data_settings_disabled(self, mock_requests, mock_logger):
        mock_body = {
            "key": "value",
        }
        m = mock_requests.post(
            DEBUG_GOOGLE_ANALYTICS_ENDPOINT,
            json=mock_body,
        )

        send_tracking_data({})
        self.assertEqual(len(m.request_history), 1)
        mock_logger.debug.assert_called_with(
            "Tracking response: %s",
            mock_body,
        )

    @requests_mock.Mocker()
    @mock.patch("django_ga_measurement_protocol.track.uuid")
    def test_track_event(self, mock_requests, mock_uuid):
        mock_uuid_value = uuid.uuid4()
        mock_uuid.uuid4.return_value = mock_uuid_value
        m = mock_requests.post(
            GOOGLE_ANALYTICS_ENDPOINT,
        )

        original_request = self.factory.get(
            "/test-url/",
            HTTP_USER_AGENT="user-agent-string",
            HTTP_REFERER="http://example.com",
        )
        common_data_params = {
            "v": [API_VERSION],
            "tid": [FAKE_GA_ID],
            "cid": [str(mock_uuid_value)],
            "uip": ["127.0.0.1"],
            "aip": ["1"],
            "ua": ["user-agent-string"],
            "dr": ["http://example.com"],
            "dl": ["http://testserver/test-url/"],
            "t": ["event"],
        }

        track_event(
            original_request,
            "test_category",
            "test_action",
        )
        request = m.request_history[0]
        data = get_request_data(request)
        self.assertEqual(
            data,
            {
                **common_data_params,
                **{
                    "ec": ["test_category"],
                    "ea": ["test_action"],
                },
            },
        )

        track_event(
            original_request,
            "test_category_2",
            "test_action_2",
            "test_label",
            "test_value",
        )
        request = m.request_history[1]
        data = get_request_data(request)
        self.assertEqual(
            data,
            {
                **common_data_params,
                **{
                    "ec": ["test_category_2"],
                    "ea": ["test_action_2"],
                    "el": ["test_label"],
                    "ev": ["test_value"],
                },
            },
        )

    @requests_mock.Mocker()
    @mock.patch("django_ga_measurement_protocol.track.uuid")
    def test_track_page_view(self, mock_requests, mock_uuid):
        mock_uuid_value = uuid.uuid4()
        mock_uuid.uuid4.return_value = mock_uuid_value
        m = mock_requests.post(
            GOOGLE_ANALYTICS_ENDPOINT,
        )

        original_request = self.factory.get(
            "/test-url/",
            HTTP_USER_AGENT="user-agent-string",
            HTTP_REFERER="http://example.com",
        )

        track_page_view(original_request)

        request = m.request_history[0]
        data = get_request_data(request)
        self.assertEqual(
            data,
            {
                "v": [API_VERSION],
                "tid": [FAKE_GA_ID],
                "cid": [str(mock_uuid_value)],
                "uip": ["127.0.0.1"],
                "aip": ["1"],
                "ua": ["user-agent-string"],
                "dr": ["http://example.com"],
                "dl": ["http://testserver/test-url/"],
                "t": ["pageview"],
            },
        )

    @requests_mock.Mocker()
    def test_unique_user_id(self, mock_requests):
        m = mock_requests.post(
            GOOGLE_ANALYTICS_ENDPOINT,
        )

        original_request = self.factory.get("/test-url/")

        user_ids = []
        for i in range(20):
            track_event(
                original_request,
                "test_category",
                "test_action",
            )
            request = m.request_history[i]
            data = get_request_data(request)
            user_ids.append(data["cid"][0])

        self.assertEqual(
            len(user_ids),
            len(set(user_ids)),
        )

        m.reset()

        user_ids = []
        for i in range(20):
            track_page_view(original_request)
            request = m.request_history[i]
            data = get_request_data(request)
            user_ids.append(data["cid"][0])

        self.assertEqual(
            len(user_ids),
            len(set(user_ids)),
        )
